package com.xcheng.scannere3;

import java.util.concurrent.CountDownLatch;

import android.util.Log;
import android.content.Context;
import android.media.AudioAttributes;
import android.media.SoundPool;

public class AudioBeep implements SoundPool.OnLoadCompleteListener {
    private static final String TAG = "AudioBeep";

    private static final float BEEP_VOLUME = 1.00f;
    private int mSoundID = 0;
    private SoundPool mSoundPool = null;

    private final CountDownLatch mCound = new CountDownLatch(1);
    private boolean mMark;

    public AudioBeep(Context ctx) {
        Log.d(TAG, "create");

        AudioAttributes attributes = new AudioAttributes.Builder()
                .setUsage(AudioAttributes.USAGE_ASSISTANCE_SONIFICATION)
                .setContentType(AudioAttributes.CONTENT_TYPE_SONIFICATION)
                .setFlags(AudioAttributes.FLAG_LOW_LATENCY|AudioAttributes.FLAG_AUDIBILITY_ENFORCED)
                .build();

        this.mSoundPool = new SoundPool.Builder()
                .setMaxStreams(4)
                .setAudioAttributes(attributes)
                .build();

        this.mSoundPool.setOnLoadCompleteListener(this);
        this.mSoundID = this.mSoundPool.load(ctx, R.raw.barcode_success2, 1);
    }

    public void play() {
        try {
            while (!this.mMark) {
                this.mCound.await();
            }

            this.mSoundPool.play(this.mSoundID, BEEP_VOLUME, BEEP_VOLUME, 1, 0, 1.0f);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public void close() {
        Log.d(TAG, "close");
        this.mSoundPool.autoPause();
        this.mSoundPool.unload(this.mSoundID);
        this.mSoundPool.release();
        this.mSoundPool = null;
        this.mSoundID = 0;
    }

    @Override
    public void onLoadComplete(SoundPool soundPool, int sampleId, int status) {
        Log.d(TAG, "onLoadComplete sampleId = " + sampleId + ", status = " + status);
        this.mMark = true;
        this.mCound.countDown();
    }
}

