package com.xcheng.scannere3;

import android.content.Context;
import android.graphics.SurfaceTexture;

//import android.hardware.Camera;

//import java.util.Array;
import java.util.Arrays;
import java.util.Locale;
import java.lang.reflect.Array;

import android.hardware.camera2.CameraCaptureSession;
import android.hardware.camera2.CameraDevice;
import android.hardware.camera2.CameraManager;
import android.hardware.camera2.CaptureRequest;
import android.hardware.camera2.CameraAccessException;
import android.hardware.camera2.CameraCharacteristics;
import android.hardware.camera2.TotalCaptureResult;
import android.view.Surface;

import android.os.SystemProperties;
import android.util.Log;
import android.util.AndroidException;

import com.xcheng.scannere3.XCScanner;

public class E3Util {
    private static final String TAG = "E3Util";

    private static final int CAM_STATE_UNINITED = 0; // Camera has not inited
    private static final int CAM_STATE_OPENED = 1;   // Camera has opened
    private static final int CAM_STATE_IDLE = 2;     // Camera session created and have no request
    private static final int CAM_STATE_PREVIEW = 3;  // Camera request processing

    private static final Object sLock = new Object();

    private static CameraCaptureSession mCameraCaptureSession = null;
    private static CameraDevice mCameraDevice = null;
    private static int mCameraStateE3 = CAM_STATE_UNINITED;
    private static boolean mAutoSetupCaptureSession = false;
    
    private static Surface mPreviewSurface = null;

    private static String mDecodeResult = null;
    private static String mDecodeSymbology = null;
    private static String mDecodeTime = null;
    private static boolean mIsDecoding = false;
    private static byte[] mPreviewBuf;
    private static long mTsStart = 0;
    private static long mTsStop = 0;

    // Hold this to avoid GC
    private static volatile SurfaceTexture mSurfaceTexture = null;

    private static XCScanner mScanner = null;
    private static XCScanner.Result mScannerResultListener = null;

    private static final XCScanner.Result mDefScannerResultListener = new XCScanner.Result(){
        @Override
        public void scanBeep() {
//            Log.d(TAG, "mDefScannerResultListener.scanBeep");
            if (mScannerResultListener != null) {
                mScannerResultListener.scanBeep();
            }
        }

        @Override
        public void scanStart() {
//            Log.d(TAG, "mDefScannerResultListener.scanStart");
            if (mScannerResultListener != null) {
                mScannerResultListener.scanStart();
            }
        }

        @Override
        public void scanResult(String sym,String content) {
//            Log.d(TAG, "mDefScannerResultListener.scanResult");

            mDecodeResult = content;
            mDecodeSymbology = sym;
            
            if (mScannerResultListener != null) {
                mScannerResultListener.scanResult(sym, content);
            }
        }
    };

    public E3Util() {
    }

    public static XCScanner getScanInstance() {
        synchronized (sLock) {
            if (mScanner == null) {
                mScanner = XCScanner.newInstance();
            }
        }
        
        return mScanner;
    }

    // Print CameraCharacteristics
    private static String formatCameraCharacteristics(CameraCharacteristics info) {
        String infoText;
        if (info != null) {
            StringBuilder infoBuilder = new StringBuilder(
                    "Camera characteristics:\n\n");

            for (CameraCharacteristics.Key<?> key : info.getKeys()) {
                infoBuilder.append(String.format(Locale.US, "%s:  ",
                        key.getName()));

                Object val = info.get(key);
                if (val.getClass().isArray()) {
                    // Iterate an array-type value
                    // Assumes camera characteristics won't have arrays of arrays as values
                    int len = Array.getLength(val);
                    infoBuilder.append("[ ");
                    for (int i = 0; i < len; i++) {
                        infoBuilder.append(String.format(Locale.US, "%s%s",
                                Array.get(val, i), (i + 1 == len) ? ""
                                        : ", "));
                    }
                    infoBuilder.append(" ]\n\n");
                } else {
                    // Single value
                    infoBuilder.append(String.format(Locale.US, "%s\n\n",
                            val.toString()));
                }
            }
            infoText = infoBuilder.toString();
        } else {
            infoText = "No info";
        }
        return infoText;
    }

    public static void setupScanResultListener(XCScanner.Result listener) {
        mScannerResultListener = listener;
    }
    
    // Camera Capture Callback
/*
    private static final CameraCaptureSession.CaptureCallback mCamCaptureCb =  new CameraCaptureSession.CaptureCallback() {
        @Override
        public void onCaptureStarted(CameraCaptureSession session, 
                                     CaptureRequest request,
                                     long timestamp,
                                     long frameNumber) {
            Log.d(TAG, "onCaptureStarted");
        }

        @Override
        public void onCaptureCompleted(CameraCaptureSession session,
                                       CaptureRequest request,
                                       TotalCaptureResult result) {
            Log.d(TAG, "onCaptureCompleted");
        }
    };
*/
    private static final CameraCaptureSession.CaptureCallback mCamCaptureCb = null;

    // Camera Sesson Callback
    private static final CameraCaptureSession.StateCallback mCamSessionCb = new CameraCaptureSession.StateCallback() {
        @Override
        public void onConfigured(CameraCaptureSession session) {
            Log.d(TAG, "CameraCaptureSession::onConfigured");
            mCameraCaptureSession = session;
        }

        @Override
        public void onConfigureFailed(CameraCaptureSession session) {
            Log.d(TAG, "CameraCaptureSession::onConfigureFailed");
        }

        @Override
        public void onReady(CameraCaptureSession session) {
            Log.d(TAG, "CameraCaptureSession::onReady");

            if (mAutoSetupCaptureSession) {
                mAutoSetupCaptureSession = false;

                Log.d(TAG, "AutoSetupCaptureSession");

                try {
                    CaptureRequest.Builder builder;
                    builder = mCameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_PREVIEW);
                    builder.addTarget(mPreviewSurface);
                    session.setRepeatingRequest(builder.build(), mCamCaptureCb, null);
                } catch (CameraAccessException e1) {
                    e1.printStackTrace();
                }
            }

            mCameraStateE3 = CAM_STATE_IDLE;
        }

        @Override
        public void onActive(CameraCaptureSession session) {
            Log.d(TAG, "CameraCaptureSession::onActive");

            mCameraStateE3 = CAM_STATE_PREVIEW;
        }

        @Override
        public void onClosed(CameraCaptureSession session) {
            Log.d(TAG, "CameraCaptureSession::onClosed");
            mCameraCaptureSession = null;
            mAutoSetupCaptureSession = false;
        }
    };

    // Camera Device StateCallback
    private static final CameraDevice.StateCallback mCamDevCb = new CameraDevice.StateCallback() {
        @Override
        public void onOpened(CameraDevice camera) {
            mCameraDevice = camera;
            Log.d(TAG, "CameraDevice::onOpened");

            mCameraStateE3 = CAM_STATE_OPENED;

            if (mPreviewSurface == null)
            {
                int resWidth = SystemProperties.getInt("vendor.debug.scan.resw", 844);
                int resHeight = SystemProperties.getInt("vendor.debug.scan.resh", 640);

                if (mSurfaceTexture == null)
                {
                    mSurfaceTexture = new SurfaceTexture(0);
                }

                mSurfaceTexture.setDefaultBufferSize(resWidth, resHeight);
                mPreviewSurface = new Surface(mSurfaceTexture);
            }
            
            try {
                mCameraDevice.createCaptureSession(Arrays.asList(mPreviewSurface), 
                                                   mCamSessionCb,
                                                   null);
            } catch (CameraAccessException e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onClosed(CameraDevice camera) {
            mCameraDevice = null;
            mCameraStateE3 = CAM_STATE_UNINITED;
        }

        @Override
        public void onError(CameraDevice camera, int error) {
            Log.d(TAG, "CameraDevice::onError, error = " + error);
            camera.close();
        }

        @Override
        public void onDisconnected(CameraDevice camera) {
            Log.d(TAG, "CameraDevice::onDisconnected");
            camera.close();
        }
    };

    // Open camera
    public static long initDecoderLibrary(Context context) {
        synchronized (sLock) {
            
            String camid = SystemProperties.get("vendor.debug.scan.camid", "2");

            Log.d(TAG, "initDecoderLibrary+");

            if (mScanner == null) {
                mScanner = XCScanner.newInstance();
                mScanner.onScanListener(mDefScannerResultListener);
            }

            CameraManager manager = (CameraManager) context.getSystemService(Context.CAMERA_SERVICE);

            try {
                //CameraCharacteristics camInfo = manager.getCameraCharacteristics(camid);
                // print CameraCharacteristics here for debug
                //if (camInfo != null) {
                //    Log.d(TAG, formatCameraCharacteristics(camInfo));
                //}

                if (mCameraDevice == null) {
                    manager.openCamera(camid, mCamDevCb, null);
                }
            } catch (CameraAccessException e) {
                e.printStackTrace();
            }

            ScanUtil.initSettings(context);

        }

        Log.d(TAG, "initDecoderLibrary-");

        return 0;
    }

    // Stop camera
    public static void releaseDecodeLibrary() {
        Log.d(TAG, "releaseDecodeLibrary+");

        mDecodeResult = null;
        mDecodeSymbology = null;
        mDecodeTime = null;

        if (mCameraCaptureSession != null)
        {
            mCameraCaptureSession.close();
        }
        mCameraCaptureSession = null;

        if (mCameraDevice != null)
        {
            mCameraDevice.close();
        }
        mCameraDevice = null;

        if (mPreviewSurface != null)
        {
            mPreviewSurface.release();
        }
        mPreviewSurface = null;

        if (mSurfaceTexture != null) {
            mSurfaceTexture.release();
        }
        mSurfaceTexture = null;

        if (mScanner != null) {
            if (mIsDecoding) {
                mScanner.stopDecode();
            }
            mScanner.deleteInstance();
            mScanner = null;
        }

        mIsDecoding = false;

        Log.d(TAG, "releaseDecodeLibrary-");
    }

    public static void doDecode() {
//        Log.d(TAG, "doDecode+");
        mTsStart = System.currentTimeMillis();

        if (mCameraCaptureSession != null && mCameraDevice != null) {
            if (mCameraStateE3 == CAM_STATE_IDLE) {
                mAutoSetupCaptureSession = false;
                Log.d(TAG, "createCaptureRequest");
                try {
                    CaptureRequest.Builder builder;
                    builder = mCameraDevice.createCaptureRequest(CameraDevice.TEMPLATE_PREVIEW);
                    builder.addTarget(mPreviewSurface);
                    mCameraCaptureSession.setRepeatingRequest(builder.build(), mCamCaptureCb, null);
                } catch (CameraAccessException e1) {
                    e1.printStackTrace();
                }
            } else if (mCameraStateE3 == CAM_STATE_OPENED) {
                // Create capture while camera opened
                mAutoSetupCaptureSession = true;
            } else if (mCameraStateE3 == CAM_STATE_PREVIEW) {
                // Camera is previewing, not need createCaptureRequest
            }
        } else {
            // Create capture while camera opened
            mAutoSetupCaptureSession = true;
        }

        if (mScanner != null) {
            mScanner.startDecode();
        }

        mDecodeResult = null;
        mDecodeSymbology = null;
        mIsDecoding = true;

//        Log.d(TAG, "doDecode-");
    }

    public static void stopDecode(){
//        Log.d(TAG, "stopDecode+");

        if (mIsDecoding) {
            if (mCameraCaptureSession != null && mCameraDevice != null) {
                Log.d(TAG, "stopRepeating");

                if (mCameraStateE3 == CAM_STATE_PREVIEW) {
                    try {
                        mCameraCaptureSession.stopRepeating();
                    } catch (AndroidException e1) {
                        e1.printStackTrace();
                    }
                }
            }

            if (mScanner != null) {
                mScanner.stopDecode();
            }
        }

        //mDecodeResult = null;
        //mDecodeSymbology = null;
        
        mDecodeTime = null;
        mIsDecoding = false;

//        Log.d(TAG, "stopDecode-");
    }

    public static boolean IsDecoding() {
        return mIsDecoding;
    }

    public static String getDecodeResult(boolean onceOnly) {
//        Log.d(TAG, "getDecodeResult+");
//        Log.d(TAG, "mDecodeResult = " + mDecodeResult);
//        Log.d(TAG, "getDecodeResult-");
        String result = mDecodeResult;
        if (onceOnly) {
            mDecodeResult = null;
        }
        return result;
    }

    public static String getDecodeSymbology(boolean onceOnly) {
//        Log.d(TAG, "getDecodeSymbology+");
//        Log.d(TAG, "mDecodeSymbology = " + mDecodeSymbology);
//        Log.d(TAG, "getDecodeSymbology-");
        String sym = mDecodeSymbology;
        if (onceOnly) {
            mDecodeSymbology = null;
        }
        return sym;
    }

    public static void setScanMode(int workmode) {
        Log.d(TAG, "setScanMode+ workmode = " + workmode);

        if (mScanner != null) {
            if (workmode >= 0) {
                mScanner.setWorkMode(workmode);
            }
        }
        
        Log.d(TAG, "setScanMode-");
    }

    // Get scan mode
    public static int getScanMode() {
        Log.d(TAG, "getScanMode+");
        int mode = XCScanner.WORKMODE_SINGLESHOT;

        if (mScanner != null) {
            mode = mScanner.getWorkMode();
        }

        Log.d(TAG, "getScanMode-, mode = " + mode);
        return mode;
    }

    public static void configViewSize(int viewsize) {
        Log.d(TAG, "configViewSize+ viewsize = " + viewsize);

        if (mScanner != null) {
            if (viewsize >= 0) {
                mScanner.configViewSize(viewsize);
            }
        }
        
        Log.d(TAG, "configViewSize-");
    }

    // Get viewsize
    public static int getViewSize() {
        Log.d(TAG, "getViewSize+");
        int viewsize = XCScanner.VIEWSIZE_100;

        if (mScanner != null) {
            viewsize = mScanner.getViewSize();
        }

        Log.d(TAG, "getViewSize-, viewsize = " + viewsize);
        return viewsize;
    }

    // Set decode timeout
    public static void setTimeout(int timeout_ms) {
        Log.d(TAG, "setTimeout+ timeout = " + timeout_ms);

        if (mScanner != null) {
            if (timeout_ms > 0) {
                mScanner.setRoundTimeout(timeout_ms);
            }
        }

        Log.d(TAG, "setTimeout-");
    }

    // Enable all code types
    public static void enableAllCodeTypes() {
        Log.d(TAG, "enableAllCodeTypes+");
        if (mScanner != null) {
            mScanner.enableAllSym();
        }
        Log.d(TAG, "enableAllCodeTypes-");
    }

    // Disable all code types
    public static void disableAllCodeTypes() {
        Log.d(TAG, "disableAllCodeTypes+");
        if (mScanner != null) {
            mScanner.disableAllSym();
        }
        Log.d(TAG, "disableAllCodeTypes-");
    }

    // Config specific code type
    public static void setCodeTypeOnOff(String symbology, boolean enable) {
        Log.d(TAG, "setCodeTypeOnOff+");

        if (mScanner == null) {
            Log.e(TAG, "mScanner null");
            return;
        }

        // Convert Application symbol string to Native symbol string
        //modify by yangzhichao for t0581 20200922 start
        //modify by yangzhichao for t0581 20201027 start
        int paramID , tag_val;
        switch(getSymID(symbology)){
            case XCScanner.E3_SYM_QRCODE:
                    paramID = XCScanner.TAG_QR_ENABLED;
                    tag_val = getNumParameter(paramID);
                    if (enable) {
                        tag_val |= (0x01 << 0);
                        tag_val |= (0x01 << 2);
                        tag_val |= (0x01 << 4);
                    } else {
                        tag_val &= (~(0x01 << 0));
                        tag_val &= (~(0x01 << 2));
                        tag_val &= (~(0x01 << 4));
                    }
                    setParameter(paramID, tag_val);
                break;
            case XCScanner.E3_SYM_DATAMATRIX:
                    paramID = XCScanner.TAG_DATAMATRIX_ENABLED;
                    tag_val = getNumParameter(paramID);
                    if (enable) {
                        tag_val |= (0x01 << 0);
                    } else {
                        tag_val &= (~(0x01 << 0));
                    }
                    setParameter(paramID, tag_val);
                break;
            case XCScanner.E3_SYM_GS1_DATABAR:
                setDecoderTag(XCScanner.TAG_RSS_14_ENABLED, enable ? 1 : 0);
                setDecoderTag(XCScanner.TAG_RSS_EXPANDED_ENABLED, enable ? 1 : 0);
                setDecoderTag(XCScanner.TAG_RSS_LIMITED_ENABLED, enable ? 1 : 0);
                break;
            default:
                mScanner.configSymOnOff(getSymID(symbology), enable ? 1 : 0);
                break;
        }


        //modify by yangzhichao for t0581 20201027 end
        Log.d(TAG, "setCodeTypeOnOff-");
    }

    //modify by yangzhichao for t0581 20201015 start
    public static int getSymID(String symbology){
        int symid = -1;
        if (symbology.equals("AZTEC")) {
            symid = XCScanner.E3_SYM_AZTEC;
        } else if (symbology.equals("C11")) {
            symid = XCScanner.E3_SYM_CODE11;
        } else if (symbology.equals("C39")) {
            symid = XCScanner.E3_SYM_CODE39;
        } else if (symbology.equals("C93")) {
            symid = XCScanner.E3_SYM_CODE93;
        } else if (symbology.equals("C128")) {
            symid = XCScanner.E3_SYM_CODE128;
        } else if (symbology.equals("Codabar")) {
            symid = XCScanner.E3_SYM_CODABAR;
        } else if (symbology.equals("CODEBLOCK F")) {
            symid = XCScanner.E3_SYM_CODABLOCK_F;
        } else if (symbology.equals("DATA MATRIX")) {
            symid = XCScanner.E3_SYM_DATAMATRIX;
        } else if (symbology.equals("EAN-8")) {
            symid = XCScanner.E3_SYM_EAN8;
        } else if (symbology.equals("EAN-13")) {
            symid = XCScanner.E3_SYM_EAN13;
        } else if (symbology.equals("GS1 DATABAR")) {
            symid = XCScanner.E3_SYM_GS1_DATABAR;
        } else if (symbology.equals("HAXIN")) {
            symid = XCScanner.E3_SYM_HANXIN;
        } else if (symbology.equals("HK25")) {
            symid = XCScanner.E3_SYM_HK25;
        } else if (symbology.equals("I25")) {
            symid = XCScanner.E3_SYM_ITF25;
        } else if (symbology.equals("MATRIX 25")) {
            symid = XCScanner.E3_SYM_MATRIX25;
        } else if (symbology.equals("MAXICODE")) {
            symid = XCScanner.E3_SYM_MAXICODE;
        } else if (symbology.equals("MSI")) {
            symid = XCScanner.E3_SYM_MSI;
        } else if (symbology.equals("MICROPDF")) {
            symid = XCScanner.E3_SYM_MICROPDF417;
        } else if (symbology.equals("PDF417")) {
            symid = XCScanner.E3_SYM_PDF417;
        } else if (symbology.equals("POSTAL")) {
            symid = XCScanner.E3_SYM_USPS_4_STATE;
        } else if (symbology.equals("QR CODE")) {
            symid = XCScanner.E3_SYM_QRCODE;
        } else if (symbology.equals("INDUSTRIAL 25")) {
            symid = XCScanner.E3_SYM_INDUSTRIAL25;
        } else if (symbology.equals("TELEPEN")) {
            symid = XCScanner.E3_SYM_TELEPEN;
        } else if (symbology.equals("TRIOPTIC")) {
            symid = XCScanner.E3_SYM_TRIOPTIC;
        } else if (symbology.equals("UPC-A")) {
            symid = XCScanner.E3_SYM_UPCA;
        } else if (symbology.equals("UPC-E")) {
            symid = XCScanner.E3_SYM_UPCE;
        } else if (symbology.equals("GS1-128")) {
            symid = XCScanner.E3_SYM_GS1_128;
        }else if (symbology.equals("GS1 DataBar Limited")) {
            symid = XCScanner.E3_SYM_GS1_LIMITED;
        }else if (symbology.equals("GS1 DataBar Expanded")) {
            symid = XCScanner.E3_SYM_GS1_EXPANDED;
        }
        return symid;
    }
    //modify by yangzhichao for t0581 20201015 end

    //modify by yangzhichao for t0581 20201016 start
    public static int getNumParameter(int paramNum) {
        return getDecoderTag(paramNum);
    }

    public static int setParameter(int paramNum, int paramVal)
    {
        return setDecoderTag(paramNum, paramVal);
    }
    //modify by yangzhichao for t0581 20201016 end

    public static int setDecoderTag(int tag, int value)
    {
        int ret = 0;
        if (mScanner == null) {
            ret = -1;
        } else {
            mScanner.configDecoderTag(tag, value);
        }

        return ret;
    }

    public static int getDecoderTag(int tag)
    {
        int ret = 0;
        if (mScanner == null) {
            ret = -1;
        } else {
            ret = mScanner.getDecoderTag(tag);
        }

        return ret;
    }

    public static void setLightAimSupport(boolean suppoted) {
        Log.d(TAG, "setLightAimSupport+");
        if (mScanner != null) {
            if (suppoted) {
                mScanner.configLights(XCScanner.LIGHTS_ID_AIM, 1);
            } else {
                mScanner.configLights(XCScanner.LIGHTS_ID_AIM, 0);
            }
        }
        Log.d(TAG, "setLightAimSupport-");
    }

    public static void setLightFlashSupport(boolean suppoted) {
        Log.d(TAG, "setLightFlashSupport+");
        if (mScanner != null) {
            if (suppoted) {
                mScanner.configLights(XCScanner.LIGHTS_ID_FLASH, 1);
            } else {
                mScanner.configLights(XCScanner.LIGHTS_ID_FLASH, 0);
            }
        }
        Log.d(TAG, "setLightFlashSupport-");
    }

    public static void setLightTestMode(int testMode) {
        Log.d(TAG, "setLightTestMode+ testMode: " + testMode);
        if (mScanner != null) {
            mScanner.setLightTestMode(testMode);
        }
        Log.d(TAG, "setLightTestMode-");
    };

}
