package com.xcheng.scannere3;

import android.app.AlertDialog;
import android.os.Bundle;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.util.Log;
import android.view.KeyEvent;
import android.widget.EditText;
import android.widget.Toast;
import android.text.TextUtils;
import android.util.Log;

import java.lang.reflect.Field;

public class MainActivity extends PreferenceActivity {

    private static final String TAG = "MainActivity";

    private static final String KEY_APP_VERSION = "app_version";

    private SharedPreferences mShare;

    private Editor editor;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.d(TAG, "onCreate +");
        super.onCreate(savedInstanceState);
        addPreferencesFromResource(R.xml.main_activity);
        setSummaryText(KEY_APP_VERSION, getLocalVersionName());
        if (!ScanTestService.isScanServiceRunning()) {
            startService(new Intent(MainActivity.this, ScanTestService.class));
        }
        mShare = ScanUtil.getSharedPreferences(MainActivity.this);
        editor = mShare.edit();
        boolean mEnableSettingLock = mShare.getBoolean(ScanSettingActivity.ENABLE_SETTING_LOCK, false);
        String mPassword  = mShare.getString(ScanSettingActivity.CHANGE_PASSWORD, "0000");
        if (mEnableSettingLock) {
            checkPassword(mPassword);
        }
        //add by yangzhichao for t0581 20201028 start
        ScanUtil.createSettingsXml(this);
        ScanUtil.initSettings(this);
        //add by yangzhichao for t0581 20201028 end
        Log.d(TAG, "onCreate -");
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume +");
        super.onResume();

        Log.d(TAG, "onResume -");
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause +");
        super.onPause();
        Log.d(TAG, "onPause -");
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy +");
        super.onDestroy();
        Log.d(TAG, "onDestroy -");
    }

    private void setSummaryText(String key, String text) {
        if (TextUtils.isEmpty(text)) {
           text = getString(R.string.text_unknown);
         }
         // some preferences may be missing
         if (findPreference(key) != null) {
             findPreference(key).setSummary(text);
         }
    }

    private void removePreferenceFromScreen(String key) {
        Preference pref = findPreference(key);
        if (pref != null) {
            getPreferenceScreen().removePreference(pref);
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        Log.d(TAG, "onKeyDown keyCode = " + keyCode + ", event = " + event);
        
        if (keyCode == KeyEvent.KEYCODE_FUNCTION
                || keyCode == KeyEvent.KEYCODE_F11
                || keyCode == KeyEvent.KEYCODE_XC_LEFT_SCAN
                || keyCode == KeyEvent.KEYCODE_XC_RIGHT_SCAN) {
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        Log.d(TAG, "onKeyUp keyCode = " + keyCode + ", event = " + event);
        
        if (keyCode == KeyEvent.KEYCODE_FUNCTION
                || keyCode == KeyEvent.KEYCODE_F11
                || keyCode == KeyEvent.KEYCODE_XC_LEFT_SCAN
                || keyCode == KeyEvent.KEYCODE_XC_RIGHT_SCAN) {
            return true;
        }
        return super.onKeyUp(keyCode, event);
    }

    private void checkPassword(final String password){
        final EditText et = new EditText(this);
        AlertDialog alertdialog = new AlertDialog.Builder(this)
        .setTitle(R.string.check_password)
        .setView(et)
        .setPositiveButton(R.string.change_password_ok, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int which) {
                    String input = et.getText().toString();
                    try{
                        Field field = dialog.getClass().getSuperclass().getDeclaredField("mShowing");
                        field.setAccessible(true);
                        if (input.equals(password)) {
                            field.set(dialog, true);
                            dialog.dismiss();
                        } else {
                            field.set(dialog, false);
                            Toast.makeText(MainActivity.this, R.string.wrong_password_tips, Toast.LENGTH_SHORT).show();
                        }
                    } catch(Exception e) {
                        e.printStackTrace();
                    }
                }
        }).create();
        alertdialog.setCancelable(false);
        alertdialog.setCanceledOnTouchOutside(false);
        alertdialog.show();
    }

    public String getLocalVersionName() {
        String localVersion = "";
        try {
            PackageInfo packageInfo = getApplicationContext()
                    .getPackageManager().getPackageInfo(getPackageName(), 0);
            localVersion = packageInfo.versionName;
        } catch (NameNotFoundException e) {
            e.printStackTrace();
        }
        return localVersion;
    }

}
