package com.xcheng.scannere3;

import android.os.Bundle;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.SwitchPreference;
import android.util.Log;
import android.content.SharedPreferences;

public class ModuleSettingActivity extends PreferenceActivity implements
        Preference.OnPreferenceChangeListener {
    private static final String TAG = "ModuleSettingActivity";

    private static final String KEY_ALL_SWITCH = "swtich_all";
    private static final String KEY_ALL_SETTING = "all_setting";

    private static final boolean VAL_ALL_SWITCH_ON = true;
    private static final boolean VAL_ALL_SWITCH_OFF = false;
    
    private static final boolean ALL_SWITCH_DEF_VAL = VAL_ALL_SWITCH_ON; // Default value if settings profile not exist.

    private SwitchPreference mAllSwitchPref;
    private SharedPreferences mSharePref;

    private Preference mAllSettingsPref;

    private boolean mEnabled = false;

    private void updateAllSwitchPrefUI() {
        boolean state = mSharePref.getBoolean(KEY_ALL_SWITCH, ALL_SWITCH_DEF_VAL);
        
        if (state == VAL_ALL_SWITCH_ON) {
            mAllSettingsPref.setEnabled(false);
        } else {
            mAllSettingsPref.setEnabled(true);
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        addPreferencesFromResource(R.xml.barcode_module_setting);

        mAllSettingsPref = findPreference(KEY_ALL_SETTING);

        mAllSwitchPref = (SwitchPreference) findPreference(KEY_ALL_SWITCH);
        mAllSwitchPref.setOnPreferenceChangeListener(this);

        mSharePref = ScanUtil.getSharedPreferences(this);
    }

    @Override
    public boolean onPreferenceChange(Preference preference, Object objValue) {
        mEnabled = (Boolean) objValue;
        Log.d(TAG, "onPreferenceChange: mEnabled = " + mEnabled);
        
        mSharePref.edit().putBoolean(KEY_ALL_SWITCH, mEnabled).commit();

        // Apply all_setting items
        ScanUtil.applyAllModulesSetting(this);

        if (mAllSettingsPref != null) {
            updateAllSwitchPrefUI();
        }
        return true;
    }

    @Override
    public void onResume() {
        super.onResume();
        boolean state = mSharePref.getBoolean(KEY_ALL_SWITCH, ALL_SWITCH_DEF_VAL);
        
        if (mAllSwitchPref != null) {
            mAllSwitchPref.setChecked(state);
        }

        if (mAllSettingsPref != null) {
            updateAllSwitchPrefUI();
        }
    }

    @Override
    public void onPause() {
        super.onPause();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }
}


