package com.xcheng.scannere3;

import android.app.Instrumentation;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.Service;
import android.content.ClipboardManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.SystemProperties;
import android.os.Vibrator;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;

import com.xcheng.scannere3.AudioBeep;
import com.xcheng.scannere3.XCScanner;

public class ScanTestService extends Service {

    private static final String TAG = "ScanTestService";

    private static final boolean mDbgLogEnable = false;

    private boolean mScanEnable = true;
    private boolean mIsInFactoryTestingMode = false; // true: ģʽ
    private static boolean mIsInScanActivity = false; // true: ɨԽ
    private ScanThreadHandler mScanTheadHandler;
    private HandlerThread mHandlerThread;

    private ToneGenerator mToneGenerator;
    private Object mToneGeneratorLock = new Object();
    private static final int TONE_LENGTH_MS = 150;
    private AudioManager mAudioManager;

    private static final int LED_NOTIFICATION_ID = 10085;
    private NotificationManager mNm = null;

    //private MediaPlayer mPlayer;
    private AudioBeep mBeeper = null;

    public static final int MSG_READ_DATA = 0x01;
    public static final int MSG_TURNOFF_SCAN_RESULT_LED = 0x04; // Turn Off
    public static final int MSG_SCAN_RESULT_NOTIFY = 0x05; // Notify Scan result

    private String mSymbologyType = null;
    private String mBarcodeData = null;

    private boolean mFirstRun = false;

    private static boolean mIsServiceRunning = false;

    // Internal test activity use this callback to get scan result
    private static XCScanner.Result mScanServiceResultCB = null;

    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            switch (msg.what) {
                case MSG_TURNOFF_SCAN_RESULT_LED:
                    cancelLedNotify();
                    break;

                case MSG_SCAN_RESULT_NOTIFY:
                    boolean isSuccess = (boolean)msg.obj;
                    scanNotification(isSuccess);
                    break;

                default:
                    break;
            }
        }
    };

    private final XCScanner.Result mScanServiceResultListener = new XCScanner.Result(){
        @Override
        public void scanBeep() {
            if (mDbgLogEnable) {
                Log.d(TAG, "mScanServiceResultListener.scanBeep");
            }

            if (mScanServiceResultCB != null) {
                mScanServiceResultCB.scanBeep();
            }
        }

        @Override
        public void scanStart() {
                if (mDbgLogEnable) {
                    Log.d(TAG, "mScanServiceResultListener.scanStart");
                }

                if (mScanServiceResultCB != null) {
                    mScanServiceResultCB.scanStart();
                }
        }

        @Override
        public void scanResult(String sym,String barcode) {
            if (mDbgLogEnable) {
                Log.d(TAG, "mScanServiceResultListener.scanResult");
            }

            if (mScanServiceResultCB != null) {
                // ԼĲԽ֪ͨص
                if (isInScanActivity()) {
                    Message msg = Message.obtain();
                    msg.what = MSG_SCAN_RESULT_NOTIFY;
                    msg.arg1 = 1;

                    if (barcode != null) {
                        msg.obj = true;
                    } else {
                        msg.obj = false;
                    }

                    mHandler.sendMessage(msg);
                }
                
                mScanServiceResultCB.scanResult(sym, barcode);
            }

            // ԼĲԽû취עصȡ
            // Notice ScanTestService: read data now!
            if (!isInScanActivity()) {
                if (!mScanTheadHandler.hasMessages(MSG_READ_DATA)) {
                    if (ScanUtil.isActivityAllowScan(ScanTestService.this)) {
                        mScanTheadHandler.sendEmptyMessage(MSG_READ_DATA);
                    }
                }
            }
        }
    };

    class ScanThreadHandler extends Handler {
        public ScanThreadHandler(Looper looper) {
            super(looper);
        }

        @Override
        public void handleMessage(Message arg) {
            Bundle bundle;
            switch (arg.what) {
                case MSG_READ_DATA:
//                    Log.d(TAG, "ScanThreadHandler::handleMessage MSG_READ_DATA");
                    String result = E3Util.getDecodeResult(true);
                    if (TextUtils.isEmpty(result)) {
                        mScanTheadHandler.sendEmptyMessage(MSG_READ_DATA);
                    } else {
                        mScanTheadHandler.removeMessages(MSG_READ_DATA);
                        if (mScanEnable || E3Util.IsDecoding()) {
                            scanNotification(true);
                            mSymbologyType = E3Util.getDecodeSymbology(true);
                            mBarcodeData = getCustomLetterCaseResult(result);
//                            Log.i(TAG, "mSymbologyType = " + mSymbologyType + " mBarcodeData = " + mBarcodeData);
                            if (ScanUtil.needModifyGsCharacter(ScanTestService.this, mBarcodeData)) {
                                mBarcodeData = ScanUtil.replaceGsCharacter(ScanTestService.this, mBarcodeData);
                            }

                            if (isInFactoryTestingMode()) {
                                // Send ScanResult to ScanTestActivity
                                sendIntentData(mSymbologyType, mBarcodeData);
                                
                                if ("CLIPBOARD_EVENT".equals(getDefaultValue(
                                        ScanSettingActivity.DATA_RECEIVE_KEY,
                                        ScanSettingActivity.DEFAULT_DATA_RECEIVE_METHOD))) {
                                    setClipboardData(new Instrumentation(), mBarcodeData);
                                }
                            } else {
                                // Send result to application via Broadcast/Keyboard/Clipboard
                                inputScanResult(mSymbologyType, mBarcodeData);
                            }
                        } else {
                            Log.d(TAG, "Scanner closed before send data");
                        }
                    }
                    break;

                default:
                    break;
            }
        }
    }

    private BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent == null) {
                return;
            }
            String action = intent.getAction();
            if (Intent.ACTION_SCREEN_ON.equals(action)) {
                Log.d(TAG, "onReceive: action = ACTION_SCREEN_ON");
                E3Util.initDecoderLibrary(getApplicationContext());
            } else if (Intent.ACTION_SCREEN_OFF.equals(action)) {
                Log.d(TAG, "onReceive: action = ACTION_SCREEN_OFF");
                E3Util.releaseDecodeLibrary();
                //E3Util.stopDecode();
            } else if (ScanUtil.ACTION_SCAN_TEST_ACTIVITY_ENTER.equals(action)) {
                Log.d(TAG, "onReceive: action = ACTION_SCAN_TEST_ACTIVITY_ENTER");
                mIsInScanActivity = true;
            } else if (ScanUtil.ACTION_SCAN_TEST_ACTIVITY_LEAVE.equals(action)) {
                Log.d(TAG, "onReceive: action = ACTION_SCAN_TEST_ACTIVITY_LEAVE");
                mIsInScanActivity = false;
                
                // ɨActivaty˳ҪʹServiceĻص
                E3Util.setupScanResultListener(mScanServiceResultListener);
            } else if (ScanUtil.ACTION_SCAN_TEST.equals(action)) {
                mIsInFactoryTestingMode = intent.getBooleanExtra(ScanUtil.FACTORY_SCAN_TEST, false);
                Log.d(TAG, "onReceive: action = ACTION_SCAN_TEST, mIsInFactoryTestingMode = " + mIsInFactoryTestingMode);
            }
        }
    };

    private void initThread() {
        mHandlerThread = new HandlerThread("ScanTestService");
        mHandlerThread.start();
        mScanTheadHandler = new ScanThreadHandler(mHandlerThread.getLooper());
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mAudioManager = (AudioManager)getSystemService(Context.AUDIO_SERVICE);
        mNm = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);

        mBeeper = new AudioBeep(this);

        initThread();
        IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_SCREEN_OFF);
        filter.addAction(Intent.ACTION_SCREEN_ON);
        filter.addAction(ScanUtil.ACTION_SCAN_TEST);
        filter.addAction(ScanUtil.ACTION_SCAN_TEST_ACTIVITY_ENTER);
        filter.addAction(ScanUtil.ACTION_SCAN_TEST_ACTIVITY_LEAVE);
        registerReceiver(mReceiver, filter);
        mFirstRun = true;
        Log.d(TAG, "onCreate");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.d(TAG, "onStartCommand: startId = " + startId);
        mScanEnable = true;

        if (mScanTheadHandler != null) {
            mScanTheadHandler.removeCallbacksAndMessages(null);
            boolean isTyping = SystemProperties.getBoolean("sys.scan.typing", false);
            Log.d(TAG, "isTyping: " + isTyping);
            E3Util.initDecoderLibrary(getApplicationContext());
            E3Util.setupScanResultListener(mScanServiceResultListener);
        }

        mIsServiceRunning = true;

        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public IBinder onBind(Intent arg0) {
        return null;
    }
    
    @Override
    public void onDestroy() {
        Log.d(TAG, "onDestroy");
        super.onDestroy();
        if (mReceiver != null) {
            unregisterReceiver(mReceiver);
        }

        if(null != mHandler) {
            mHandler.removeMessages(MSG_TURNOFF_SCAN_RESULT_LED);
        }

        if (mScanTheadHandler != null) {
            mScanTheadHandler.removeCallbacksAndMessages(null);
        }
        E3Util.releaseDecodeLibrary();
        quitThread();


        if (mBeeper != null) {
            mBeeper.close();
            mBeeper = null;
        }

        mIsInFactoryTestingMode = false;
        mIsInScanActivity = false;

        mIsServiceRunning = false;
    }

    private void quitThread() {
        if (mHandlerThread != null) {
            boolean mIsQuit = mHandlerThread.quit();
            Log.d(TAG, "quitThread: mIsQuit = " + mIsQuit);
        }
    }

    private void inputScanResult(String symbology, String result) {
        Log.i(TAG, "inputScanResult, symbology = " + symbology + " result = " + result);

        Instrumentation inst = new Instrumentation();
        selectDataReceiveMethod(inst, symbology, result);
    }

    /**
     * @param symbology Code type,for example:C128 C39
     * @param barcode Code scanning results
     */
    private void sendIntentData(String symbology, String barcode) {
        Log.i(TAG, "sendIntentData, SCAN_DECODING_BROADCAST, symbology = " + symbology + " barcode = " + barcode);

        String prefixstr = getPrefixString();
        String suffixstr = getSuffixString();

        Intent intent = new Intent(ScanUtil.SCAN_DECODING_BROADCAST);
        intent.putExtra(ScanUtil.SCAN_SYMBOLOGY_TYPE, symbology);
        intent.putExtra(ScanUtil.SCAN_DECODING_DATA, prefixstr+barcode+suffixstr);
        sendBroadcast(intent);
    }

    private void sendKeyboardData(Instrumentation inst, String result) {
        char[] chars = result.toCharArray();
        String delayTimeValue = getDefaultValue(ScanSettingActivity.TYPING_DELAY_KEY, "0");
        String userDefValue = getDefaultValue(ScanSettingActivity.USER_DEFINITION_KEY, "0");
        int delayTime = Integer.parseInt(delayTimeValue);
        if (delayTime == 1) {
            delayTime = Integer.parseInt(userDefValue);
        }
        Log.i(TAG, "sendKeyboardData: delayTime = " + delayTime);
		SystemProperties.set("sys.scan.typing", "true");
        sendPrefixKeyboardData(inst);
        for(int i = 0; i < chars.length; i++) {
            Log.i(TAG, "sendKeyboardData: result = " + String.valueOf(chars[i]));
            inst.sendStringSync(String.valueOf(chars[i]));
            try {
                Thread.currentThread().sleep(delayTime);
            } catch (InterruptedException e) {
                 e.printStackTrace();
            }
        }
        sendSuffixKeyboardData(inst);
        SystemProperties.set("sys.scan.typing", "false");
    }

    private void setClipboardData(Instrumentation inst, String result) {
        ClipboardManager copy = (ClipboardManager)getSystemService(Context.CLIPBOARD_SERVICE);
        String prefixstr = getPrefixString();
        String suffixstr = getSuffixString();
        copy.setText(prefixstr+result+suffixstr);
        inst.sendCharacterSync(KeyEvent.KEYCODE_PASTE);
    }

    private void selectDataReceiveMethod(Instrumentation inst, String symbology, String result) {
        String value = getDefaultValue(ScanSettingActivity.DATA_RECEIVE_KEY,
                ScanSettingActivity.DEFAULT_DATA_RECEIVE_METHOD);

        Log.i(TAG, "selectDataReceiveMethod, symbology = " + symbology + 
                   " result = " + result + 
                   " DATA_RECEIVE_KEY = " + value);

        if ("KEYBOARD_EVENT".equals(value)) {
            // send data to application via KeyBoard
            sendKeyboardData(inst, result);
        } else if ("CLIPBOARD_EVENT".equals(value)) {
            // send data to application via clipboard
            setClipboardData(inst, result);
        } else if ("BROADCAST_EVENT".equals(value)) {
            // send data to application via broadcast
            sendIntentData(symbology, result);
        } else if ("NONE".equals(value)) {
            Log.i(TAG, "Scan data dropped");
        }
    }

    private void sendPrefixKeyboardData(Instrumentation inst){
        String mPrefixChar1  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.PREFIX_CHAR1_KEY,ScanSettingActivity.EMPTY);
        String mPrefixChar2  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.PREFIX_CHAR2_KEY,ScanSettingActivity.EMPTY);
        sendSpecialCharacters(inst,mPrefixChar1,false);
        sendSpecialCharacters(inst,mPrefixChar2,false);
    }

    private String getPrefixString(){
        String mPrefixChar1  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.PREFIX_CHAR1_KEY,ScanSettingActivity.EMPTY);
        String mPrefixChar2  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.PREFIX_CHAR2_KEY,ScanSettingActivity.EMPTY);
        String ret = "";

        // Process PrefixChar1
        if(("Empty").equals(mPrefixChar1)){
            ret = "";
        }else if(("TAB").equals(mPrefixChar1)){
            ret = "\t"; // TAB
        }else if(("ENTER").equals(mPrefixChar1)){
            ret = "\n"; // Enter
        }else if(("SPACE").equals(mPrefixChar1)){
            ret = " ";
        }else{
            ret = mPrefixChar1;
        }

        // Process PrefixChar2
        if(("Empty").equals(mPrefixChar2)){
            ret += "";
        }else if(("TAB").equals(mPrefixChar2)){
            ret += "\t"; // TAB
        }else if(("ENTER").equals(mPrefixChar2)){
            ret += "\n"; // Enter
        }else if(("SPACE").equals(mPrefixChar2)){
            ret += " ";
        }else{
            ret += mPrefixChar2;
        }

        // ģʽǰ׺
        if (isInFactoryTestingMode()) {
            ret = "";
        }

        return ret;
    }

    private void sendSuffixKeyboardData(Instrumentation inst){
        String mSuffixChar1  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.SUFFIX_CHAR1_KEY,"ENTER");
        String mSuffixChar2  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.SUFFIX_CHAR2_KEY,ScanSettingActivity.EMPTY);
        sendSpecialCharacters(inst,mSuffixChar1,true);
        sendSpecialCharacters(inst,mSuffixChar2,true);
    }

    private String getSuffixString(){
        String mSuffixChar1  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.SUFFIX_CHAR1_KEY,"ENTER");
        String mSuffixChar2  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.SUFFIX_CHAR2_KEY,ScanSettingActivity.EMPTY);

        String ret = "";

        // Process SuffixChar1
        if(("Empty").equals(mSuffixChar1)){
            ret = "";
        }else if(("TAB").equals(mSuffixChar1)){
            ret = "\t"; // TAB
        }else if(("ENTER").equals(mSuffixChar1)){
            ret = "\n"; // Enter
        }else if(("SPACE").equals(mSuffixChar1)){
            ret = " ";
        }else{
            ret = mSuffixChar1;
        }

        // Process SuffixChar2
        if(("Empty").equals(mSuffixChar2)){
            ret += "";
        }else if(("TAB").equals(mSuffixChar2)){
            ret += "\t"; // TAB
        }else if(("ENTER").equals(mSuffixChar2)){
            ret += "\n"; // Enter
        }else if(("SPACE").equals(mSuffixChar2)){
            ret += " ";
        }else{
            ret += mSuffixChar2;
        }

        // ģʽǰ׺
        if (isInFactoryTestingMode()) {
            ret = "";
        }

        return ret;
    }

    private void sendSpecialCharacters(Instrumentation inst,String specialStr,boolean sendTabEnter){
        if(("Empty").equals(specialStr)){
        }else if(("TAB").equals(specialStr)){
            if(sendTabEnter) inst.sendKeyDownUpSync(KeyEvent.KEYCODE_TAB);
        }else if(("ENTER").equals(specialStr)){
            if(sendTabEnter) inst.sendKeyDownUpSync(KeyEvent.KEYCODE_ENTER);
        }else if(("SPACE").equals(specialStr)){
            inst.sendKeyDownUpSync(KeyEvent.KEYCODE_SPACE);
        }else{
            inst.sendStringSync(specialStr);
        }
    }

    private String getCustomLetterCaseResult(String result){
        String mLetterCase  = ScanUtil.getSharedPreferences(ScanTestService.this).getString(ScanSettingActivity.LETTER_CASE_KEY,"NONE_CASE");
        switch(mLetterCase){
            case "NONE_CASE":
                break;
            case "UPPER_CASE":
                result = result.toUpperCase();
                break;
            case "LOWER_CASE":
                result = result.toLowerCase();
                break;
            default:
                break;
        }
        return result;
    }

    private void playBeep() {
//        Log.i(TAG, "playBeep");

        if (mBeeper != null) {
            mBeeper.play();
        }
    }

    private void vibrate() {
        Vibrator vibrator = (Vibrator)this.getSystemService(Context.VIBRATOR_SERVICE);
        int ringerMode = mAudioManager.getRingerMode();
        if ((ringerMode == AudioManager.RINGER_MODE_SILENT)) {
            return;
        }
        if (vibrator != null) {
            vibrator.vibrate(200);
        }
    }

    private void scanNotification(boolean isSuccess) {
        String value;

//        Log.i(TAG, "scanNotification(" + isSuccess + ")");
        
        if (isSuccess) {
            value = ScanUtil.getSharedPreferences(ScanTestService.this)
                    .getString(ScanSettingActivity.SUCCESS_NOTIFICATION_KEY,
                            "Sound");
        } else {
            value = ScanUtil.getSharedPreferences(ScanTestService.this)
                    .getString(ScanSettingActivity.FAIL_NOTIFICATION_KEY,
                            "Mute");
        }

        //Notify led green light
        if(isSuccess) {
            sendLedNotification(0xff00ff00);
            if(null != mHandler) {
                mHandler.removeMessages(MSG_TURNOFF_SCAN_RESULT_LED);
                mHandler.sendEmptyMessageDelayed(MSG_TURNOFF_SCAN_RESULT_LED, 100);
            }
        }

        if ("Sound".equals(value) || "Sound/Vib".equals(value)) {
            playBeep();
        }
        if ("Vib".equals(value) || "Sound/Vib".equals(value)) {
            vibrate();
        }
    }

    public String getDefaultValue(String key, String defaultValue) {
        String value = ScanUtil.getSharedPreferences(this).getString(key,
                defaultValue);
        return value;
    }

    private boolean isInFactoryTestingMode() {
        Log.d(TAG, "mIsInFactoryTestingMode = " + mIsInFactoryTestingMode);
        return mIsInFactoryTestingMode;
    }

    public static boolean isInScanActivity() {
//        Log.d(TAG, "mIsInScanActivity = " + mIsInScanActivity);
        return mIsInScanActivity;
    }

    public static boolean isScanServiceRunning() {
//        Log.d(TAG, "mIsServiceRunning = " + mIsServiceRunning);
        return mIsServiceRunning;
    }

    public static void setupScanServiceResultListener(XCScanner.Result listener) {
        mScanServiceResultCB = listener;
    }

    private void sendLedNotification(int color) {
        if(null != mNm) {
            Notification notify = new Notification.Builder(this)
                //.setSmallIcon(null)
                .setSmallIcon(R.mipmap.ic_launcher)
                .setLights(color, 1, 0)
                .build();

            mNm.notify(LED_NOTIFICATION_ID, notify);
        }
    }

    private void cancelLedNotify() {
        if(null != mNm) {
            mNm.cancel(LED_NOTIFICATION_ID);
        }
    }
}

