package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;


public class DataMatrixSetting extends BaseSetting {

    private static final String TAG = "DataMatrixSetting";

    private static final String KEY_MIN = "datamatrix_min";
    private static final String KEY_MAX = "datamatrix_max";

    private static final int MAX_LENGTH = 3116;
    private static final int MIN_LENGTH = 1;

    private Preference mMaxPref;
    private Preference mMinPref;

    private static final String KEY_DM_REVERSE = "datamatrix_reverse";
    private static final String KEY_DM_MIRROR = "datamatrix_mirror";

    private SwitchPreference mDMReversePref;
    private SwitchPreference mDMMirrorPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        int dmEnableValue = getDecoderTagValue(XCScanner.TAG_DATAMATRIX_ENABLED);
        Log.d(TAG, "dmEnableValue = " + dmEnableValue);

        mDMReversePref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_DM_REVERSE, getString(R.string.text_reverse_enable), XCScanner.TAG_DATAMATRIX_ENABLED);

        // Update checkbox via decoder status
        if ((dmEnableValue & (0x01<<1)) == 0) {
            mDMReversePref.setChecked(false);
        } else {
            mDMReversePref.setChecked(true);
        }

        mDMReversePref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                int tagValue = getDecoderTagValue(XCScanner.TAG_DATAMATRIX_ENABLED);
                Editor editor = BaseSetting.getSharedPreferences(DataMatrixSetting.this).edit();
                editor.putInt(KEY_DM_REVERSE, checked ? 1 : 0);
                editor.commit();

                if (checked) {
                    tagValue |= (0x01<<1);
                } else {
                    tagValue &= (~(0x01<<1));
                }

                setDecoderTag(XCScanner.TAG_DATAMATRIX_ENABLED, tagValue);
                return true;
            }
        });

        mDMMirrorPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_DM_MIRROR, getString(R.string.text_mirror_enable), XCScanner.TAG_DATAMATRIX_ENABLED);

        // Update checkbox via decoder status
        if ((dmEnableValue & (0x01<<6)) == 0) {
            mDMMirrorPref.setChecked(false);
        } else {
            mDMMirrorPref.setChecked(true);
        }

        mDMMirrorPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                int tagValue = getDecoderTagValue(XCScanner.TAG_DATAMATRIX_ENABLED);
                Editor editor = BaseSetting.getSharedPreferences(DataMatrixSetting.this).edit();
                editor.putInt(KEY_DM_MIRROR, checked ? 1 : 0);
                editor.commit();

                if (checked) {
                    tagValue |= (0x01<<6);
                } else {
                    tagValue &= (~(0x01<<6));
                }

                setDecoderTag(XCScanner.TAG_DATAMATRIX_ENABLED, tagValue);
                return true;
            }
        });

        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_DATAMATRIX_MAX_LENGTH);
        int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue2 = getDecoderTagValue(XCScanner.TAG_DATAMATRIX_MAX_LENGTH);
        Log.d(TAG, "maxValue = " + maxValue + ", maxValue2 = " + maxValue2);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        } else {
            mMaxPref.setSummary(String.valueOf(MAX_LENGTH));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(DataMatrixSetting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_DATAMATRIX_MAX_LENGTH);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_DATAMATRIX_MIN_LENGTH);
        int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue2 = getDecoderTagValue(XCScanner.TAG_DATAMATRIX_MIN_LENGTH);
        Log.d(TAG, "minValue = " + minValue + "minValue2 = " + minValue2);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(DataMatrixSetting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_DATAMATRIX_MIN_LENGTH);
                return true;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
