package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;

public class Ean8Setting extends BaseSetting {

    private static final String TAG = "Ean8Setting";

    private static final String KEY_TRANSMIT_CHECK = "ean8_transmit_check";
    private static final String KEY_2CHAR_ADDENDA = "ean8_2char_addenda";
    private static final String KEY_5CHAR_ADDENDA = "ean8_5char_addenda";
    private static final String KEY_REQUIRED_ADDENDA = "ean8_required_addenda";
    private static final String KEY_SEPARATOR_ADDENDA = "ean8_separator_addenda";

    private SwitchPreference mTransmitCheck;
    private SwitchPreference mTwoCharAddenda;
    private SwitchPreference mFiveCharAddenda;
    private SwitchPreference mRequiredAddenda;
    private SwitchPreference mSeparatorAddenda;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mTransmitCheck = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT_CHECK, getString(R.string.transmit_check_digit), XCScanner.TAG_EAN8_CHECK_DIGIT_TRANSMIT);
        mTransmitCheck.setChecked(getDecoderTagValue(XCScanner.TAG_EAN8_CHECK_DIGIT_TRANSMIT) == 1);

        mTransmitCheck.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Ean8Setting.this).edit();
                editor.putInt(KEY_TRANSMIT_CHECK, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_EAN8_CHECK_DIGIT_TRANSMIT, checked ? 1 : 0);
                return true;
            }
        });

        mTwoCharAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_2CHAR_ADDENDA, getString(R.string.digit_addenda_2), XCScanner.TAG_EAN8_2CHAR_ADDENDA_ENABLED);
        mTwoCharAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_EAN8_2CHAR_ADDENDA_ENABLED) == 1);

        mTwoCharAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Ean8Setting.this).edit();
                editor.putInt(KEY_2CHAR_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_EAN8_2CHAR_ADDENDA_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mFiveCharAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_5CHAR_ADDENDA, getString(R.string.digit_addenda_5), XCScanner.TAG_EAN8_5CHAR_ADDENDA_ENABLED);
        mFiveCharAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_EAN8_5CHAR_ADDENDA_ENABLED) == 1);

        mFiveCharAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Ean8Setting.this).edit();
                editor.putInt(KEY_5CHAR_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_EAN8_5CHAR_ADDENDA_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mRequiredAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_REQUIRED_ADDENDA, getString(R.string.addenda_required), XCScanner.TAG_EAN8_ADDENDA_REQUIRED);
        mRequiredAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_EAN8_ADDENDA_REQUIRED) == 1);

        mRequiredAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Ean8Setting.this).edit();
                editor.putInt(KEY_REQUIRED_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_EAN8_ADDENDA_REQUIRED, checked ? 1 : 0);
                return true;
            }
        });

        mSeparatorAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_SEPARATOR_ADDENDA, getString(R.string.addenda_add_separator), XCScanner.TAG_EAN8_ADDENDA_SEPARATOR);
        mSeparatorAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_EAN8_ADDENDA_SEPARATOR) == 1);

        mSeparatorAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Ean8Setting.this).edit();
                editor.putInt(KEY_SEPARATOR_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_EAN8_ADDENDA_SEPARATOR, checked ? 1 : 0);
                return true;
            }
        });

    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
