package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;

public class Gs1DataBarSetting extends BaseSetting {

    private static final String TAG = "Gs1DataBarSetting";

    private static final int LIMITED_PARM = 339;
    private static final int LIMITED_LEVEL_PARM = 728;
    private static final int EXPANDED_PARM = 340;

    private static final String KEY_LIMITED = "gs1_limited";
    private static final String KEY_EXPANDED = "gs1_expanded";

    private SwitchPreference mLimitedPref;
    private SwitchPreference mExpandedPref;

    private Preference mMaxPref;
    private Preference mMinPref;

    private static final String KEY_MIN = "gs1expanded_min";
    private static final String KEY_MAX = "gs1expanded_max";

    private static final int MAX_LENGTH = 128;
    private static final int MIN_LENGTH = 1;
    private static final int DFAULT_MIN_LENGTH = 2;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mExpandedPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_EXPANDED, getString(R.string.text_gs1_expanded), XCScanner.TAG_RSS_EXPANDED_ENABLED);
        //mExpandedPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_EXPANDED, -1) == 1);
        mExpandedPref.setChecked(getDecoderTagValue(XCScanner.TAG_RSS_EXPANDED_ENABLED) == 1);

        mExpandedPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Gs1DataBarSetting.this).edit();
                editor.putInt(KEY_EXPANDED, checked ? 1 : 0);
                editor.commit();
                //setParameter(EXPANDED_PARM, checked ? 1 : 0);
                setDecoderTag(XCScanner.TAG_RSS_EXPANDED_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mLimitedPref = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_LIMITED, getString(R.string.text_gs1_limited), XCScanner.TAG_RSS_LIMITED_ENABLED);
        //mLimitedPref.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_LIMITED, -1) == 1);
        mLimitedPref.setChecked(getDecoderTagValue(XCScanner.TAG_RSS_LIMITED_ENABLED) == 1);

        mLimitedPref.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(Gs1DataBarSetting.this).edit();
                editor.putInt(KEY_LIMITED, checked ? 1 : 0);
                editor.commit();
                //setParameter(LIMITED_PARM, checked ? 1 : 0);
                setDecoderTag(XCScanner.TAG_RSS_LIMITED_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        final String maxTitle = getString(R.string.setting_max_expanded_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_RSS_EXPANDED_MAX_LENGTH/*MAX_PARM*/);
        int maxValue = BaseSetting.getSharedPreferences(Gs1DataBarSetting.this).getInt(KEY_MAX, -1);
        int maxValue2 = getDecoderTagValue(XCScanner.TAG_RSS_EXPANDED_MAX_LENGTH);
        Log.d(TAG, "maxValue = " + maxValue + ", maxValue2 = " + maxValue2);

        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(Gs1DataBarSetting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_RSS_EXPANDED_MAX_LENGTH/*MAX_PARM*/);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_expanded_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_RSS_EXPANDED_MIN_LENGTH/*MIN_PARM*/);
        int minValue = BaseSetting.getSharedPreferences(Gs1DataBarSetting.this).getInt(KEY_MIN, -1);
        int minValue2 = getDecoderTagValue(XCScanner.TAG_RSS_EXPANDED_MIN_LENGTH);
        Log.d(TAG, "minValue = " + minValue + ", minValue2 = " + minValue2);

        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(DFAULT_MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(Gs1DataBarSetting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_RSS_EXPANDED_MIN_LENGTH/*MIN_PARM*/);
                return true;
            }
        });

    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
