package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;

public class Pdf417Setting extends BaseSetting {

    private static final String TAG = "Pdf417Setting";

    private static final String KEY_MIN = "pdf417_min";
    private static final String KEY_MAX = "pdf417_max";

    private static final int MAX_LENGTH = 366;
    private static final int MIN_LENGTH = 1;

    private Preference mMaxPref;
    private Preference mMinPref;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        final String maxTitle = getString(R.string.setting_max_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMaxPref = createPreference(this, TYPE_PREFERENCE, KEY_MAX, maxTitle, XCScanner.TAG_MICROPDF_MAX_LENGTH);
        int maxValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MAX, -1);
        int maxValue2 = getDecoderTagValue(XCScanner.TAG_MICROPDF_MAX_LENGTH);
        Log.d(TAG, "maxValue = " + maxValue + ", maxValue2 = " + maxValue2);
        if (maxValue != -1){
            mMaxPref.setSummary(Integer.toString(maxValue));
        } else {
            mMaxPref.setSummary(String.valueOf(MAX_LENGTH));
        }
        mMaxPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MAX);
                createMaxLengthDialog(Pdf417Setting.this, preference, maxTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_MICROPDF_MAX_LENGTH);
                return true;
            }
        });

        final String minTitle = getString(R.string.setting_min_length) + " (" + MIN_LENGTH + "-" + MAX_LENGTH + ")";
        mMinPref = createPreference(this, TYPE_PREFERENCE, KEY_MIN, minTitle, XCScanner.TAG_MICROPDF_MIN_LENGTH);
        int minValue = BaseSetting.getSharedPreferences(this).getInt(KEY_MIN, -1);
        int minValue2 = getDecoderTagValue(XCScanner.TAG_MICROPDF_MIN_LENGTH);
        Log.d(TAG, "minValue = " + minValue + "minValue2 = " + minValue2);
        if (minValue != -1){
            mMinPref.setSummary(Integer.toString(minValue));
        } else {
            mMinPref.setSummary(String.valueOf(MIN_LENGTH));
        }
        mMinPref.setOnPreferenceClickListener(new OnPreferenceClickListener() {
            @Override
            public boolean onPreferenceClick(Preference preference) {
                preference.setKey(KEY_MIN);
                createMinLengthDialog(Pdf417Setting.this, preference, minTitle,
                        MAX_LENGTH, MIN_LENGTH, XCScanner.TAG_MICROPDF_MIN_LENGTH);
                return true;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
