package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;
import com.xcheng.scannere3.XCScanner;

public class UpcASetting extends BaseSetting {

    private static final String TAG = "UpcASetting";

    private static final String KEY_TRANSMIT_CHECK = "upca_transmit_check";
    private static final String KEY_NUMBER_SYSTEM = "upca_number_system";
    private static final String KEY_2CHAR_ADDENDA = "upca_2char_addenda";
    private static final String KEY_5CHAR_ADDENDA = "upca_5char_addenda";
    private static final String KEY_REQUIRED_ADDENDA = "upca_required_addenda";
    private static final String KEY_SEPARATOR_ADDENDA = "upca_separator_addenda";
    private static final String KEY_COUNTRY_CODE = "upca_country_code";

    private SwitchPreference mTransmitCheck;
    private SwitchPreference mNumberSystemCheck;
    private SwitchPreference mTwoCharAddenda;
    private SwitchPreference mFiveCharAddenda;
    private SwitchPreference mRequiredAddenda;
    private SwitchPreference mSeparatorAddenda;
    private SwitchPreference mCountryCode;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mTransmitCheck = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT_CHECK, getString(R.string.transmit_check_digit), XCScanner.TAG_UPCA_CHECK_DIGIT_TRANSMIT);
        mTransmitCheck.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_CHECK_DIGIT_TRANSMIT) == 1);

        mTransmitCheck.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_TRANSMIT_CHECK, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_CHECK_DIGIT_TRANSMIT, checked ? 1 : 0);
                return true;
            }
        });

        mNumberSystemCheck = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_NUMBER_SYSTEM, getString(R.string.number_system), XCScanner.TAG_UPCA_NUMBER_SYSTEM_TRANSMIT);
        mNumberSystemCheck.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_NUMBER_SYSTEM_TRANSMIT) == 1);

        mNumberSystemCheck.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_NUMBER_SYSTEM, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_NUMBER_SYSTEM_TRANSMIT, checked ? 1 : 0);
                return true;
            }
        });

        mTwoCharAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_2CHAR_ADDENDA, getString(R.string.digit_addenda_2), XCScanner.TAG_UPCA_2CHAR_ADDENDA_ENABLED);
        mTwoCharAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_2CHAR_ADDENDA_ENABLED) == 1);

        mTwoCharAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_2CHAR_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_2CHAR_ADDENDA_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mFiveCharAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_5CHAR_ADDENDA, getString(R.string.digit_addenda_5), XCScanner.TAG_UPCA_5CHAR_ADDENDA_ENABLED);
        mFiveCharAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_5CHAR_ADDENDA_ENABLED) == 1);

        mFiveCharAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_5CHAR_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_5CHAR_ADDENDA_ENABLED, checked ? 1 : 0);
                return true;
            }
        });

        mRequiredAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_REQUIRED_ADDENDA, getString(R.string.addenda_required), XCScanner.TAG_UPCA_ADDENDA_REQUIRED);
        mRequiredAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_ADDENDA_REQUIRED) == 1);

        mRequiredAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_REQUIRED_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_ADDENDA_REQUIRED, checked ? 1 : 0);
                return true;
            }
        });

        mSeparatorAddenda = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_SEPARATOR_ADDENDA, getString(R.string.addenda_add_separator), XCScanner.TAG_UPCA_ADDENDA_SEPARATOR);
        mSeparatorAddenda.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_ADDENDA_SEPARATOR) == 1);

        mSeparatorAddenda.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_SEPARATOR_ADDENDA, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_ADDENDA_SEPARATOR, checked ? 1 : 0);
                return true;
            }
        });

        mCountryCode = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_COUNTRY_CODE, getString(R.string.country_code), XCScanner.TAG_UPCA_ADD_COUNTRY_CODE);
        mCountryCode.setChecked(getDecoderTagValue(XCScanner.TAG_UPCA_ADD_COUNTRY_CODE) == 1);

        mCountryCode.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcASetting.this).edit();
                editor.putInt(KEY_COUNTRY_CODE, checked ? 1 : 0);
                editor.commit();
                setDecoderTag(XCScanner.TAG_UPCA_ADD_COUNTRY_CODE, checked ? 1 : 0);
                return true;
            }
        });

    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
