package com.xcheng.scannere3.codesetting;

import android.os.Bundle;
import android.content.Context;
import android.content.SharedPreferences.Editor;
import android.preference.ListPreference;
import android.preference.SwitchPreference;
import android.preference.Preference;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.util.Log;
import android.text.TextUtils;
import com.xcheng.scannere3.R;

public class UpcSetting extends BaseSetting {

    private static final String TAG = "UpcSetting";

    private static final int EAN_JAN_8_EXTEND_PARM = 39;
    private static final int TRANSMIT_UPC_A_PARM = 40;
    private static final int TRANSMIT_UPC_E_PARM = 41;
    private static final int CONVERT_UPC_E_PARM = 37;
    private static final int PREAMBLE_UPC_A_PARM = 34;
    private static final int PREAMBLE_UPC_E_PARM = 35;
    private static final int DECODE_SUPL_PARM = 16;

    private static final String KEY_EAN_JAN_8_EXTEND = "EanJan8_extend";
    private static final String KEY_TRANSMIT_UPC_A = "upc_a_transmit";
    private static final String KEY_TRANSMIT_UPC_E = "upc_e_transmit";
    private static final String KEY_CONVERT_UPC_E = "upc_e_convert";
    private static final String KEY_PREAMBLE_UPC_A = "upc_a_preamble";
    private static final String KEY_PREAMBLE_UPC_E = "upc_e_preamble";
    private static final String KEY_DECODE_SUPL = "decode_supplemental";

    private SwitchPreference mEanJan8Extend;
    private SwitchPreference mUpcATransmit;
    private SwitchPreference mUpcETransmitUpcE;
    private SwitchPreference mConvertUpcE;
    private ListPreference mPreambleUpcA;
    private ListPreference mPreambleUpcE;
    private ListPreference mDecodeSupl;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        mEanJan8Extend = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_EAN_JAN_8_EXTEND, getString(R.string.text_ean8_extend), EAN_JAN_8_EXTEND_PARM);
        mEanJan8Extend.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_EAN_JAN_8_EXTEND, -1) == 1);
        mEanJan8Extend.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_EAN_JAN_8_EXTEND, checked ? 1 : 0);
                editor.commit();
                setParameter(EAN_JAN_8_EXTEND_PARM, checked ? 1 : 0);
                return true;
            }
        });

        mUpcATransmit = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT_UPC_A, getString(R.string.transmit_upc, getString(R.string.text_upc_a)), TRANSMIT_UPC_A_PARM);
        mUpcATransmit.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_TRANSMIT_UPC_A, -1) == 1);
        mUpcATransmit.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_TRANSMIT_UPC_A, checked ? 1 : 0);
                editor.commit();
                setParameter(TRANSMIT_UPC_A_PARM, checked ? 1 : 0);
                return true;
            }
        });

        mUpcETransmitUpcE = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_TRANSMIT_UPC_E, getString(R.string.transmit_upc, getString(R.string.text_upc_e)), TRANSMIT_UPC_E_PARM);
        mUpcETransmitUpcE.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_TRANSMIT_UPC_E, -1) == 1);
        mUpcETransmitUpcE.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_TRANSMIT_UPC_E, checked ? 1: 0);
                editor.commit();
                setParameter(TRANSMIT_UPC_E_PARM, checked ? 1 : 0);
                return true;
            }
        });

        mConvertUpcE = (SwitchPreference) createPreference(this, TYPE_SWTICH,
                KEY_CONVERT_UPC_E, getString(R.string.convert_upce_to_upca), CONVERT_UPC_E_PARM);
        mConvertUpcE.setChecked(BaseSetting.getSharedPreferences(this).getInt(KEY_CONVERT_UPC_E, -1) == 1);
        mConvertUpcE.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                boolean checked = (Boolean) objValue;
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_CONVERT_UPC_E, checked ? 1 : 0);
                editor.commit();
                setParameter(CONVERT_UPC_E_PARM, checked ? 1 : 0);
                return true;
            }
        });

        mPreambleUpcA = (ListPreference) createPreference(this, TYPE_LIST,
                KEY_PREAMBLE_UPC_A, getString(R.string.text_upc_a_preamble), PREAMBLE_UPC_A_PARM);
        mPreambleUpcA.setEntries(R.array.upc_preamble_entries);
        mPreambleUpcA.setEntryValues(R.array.upc_preamble_values);
        mPreambleUpcA.setDialogTitle(R.string.text_upc_a_preamble);
        int valueUpcA = BaseSetting.getSharedPreferences(this).getInt(KEY_PREAMBLE_UPC_A, -1);
        if (valueUpcA != -1){
            mPreambleUpcA.setValueIndex(valueUpcA);
            updateListPreferenceSummary(mPreambleUpcA, valueUpcA);
        }
        mPreambleUpcA.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_PREAMBLE_UPC_A, new Integer((String) objValue));
                editor.commit();
                setParameter(PREAMBLE_UPC_A_PARM, new Integer((String) objValue));
                updateListPreferenceSummary((ListPreference)preference,
                        Long.parseLong((String) objValue));
                return true;
            }
        });

        mPreambleUpcE = (ListPreference) createPreference(this, TYPE_LIST,
                KEY_PREAMBLE_UPC_E, getString(R.string.text_upc_e_preamble), PREAMBLE_UPC_E_PARM);
        mPreambleUpcE.setEntries(R.array.upc_preamble_entries);
        mPreambleUpcE.setEntryValues(R.array.upc_preamble_values);
        mPreambleUpcE.setDialogTitle(R.string.text_upc_e_preamble);
        int valueUpcE = BaseSetting.getSharedPreferences(this).getInt(KEY_PREAMBLE_UPC_E, -1);
        if (valueUpcE != -1){
            mPreambleUpcE.setValueIndex(valueUpcE);
            updateListPreferenceSummary(mPreambleUpcE, valueUpcE);
        }
        mPreambleUpcE.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_PREAMBLE_UPC_E, new Integer((String) objValue));
                editor.commit();
                setParameter(PREAMBLE_UPC_E_PARM, new Integer((String) objValue));
                updateListPreferenceSummary((ListPreference)preference,
                        Long.parseLong((String) objValue));
                return true;
            }
        });

        mDecodeSupl = (ListPreference) createPreference(this, TYPE_LIST,
                KEY_DECODE_SUPL, getString(R.string.text_decode_supplemental), DECODE_SUPL_PARM);
        mDecodeSupl.setEntries(R.array.ean_upc_supplementals_entries);
        mDecodeSupl.setEntryValues(R.array.ean_upc_supplementals_values);
        mDecodeSupl.setDialogTitle(R.string.text_decode_supplemental);
        int value = BaseSetting.getSharedPreferences(this).getInt(KEY_DECODE_SUPL, -1);
        if (value != -1){
            mDecodeSupl.setValueIndex(value);
            updateListPreferenceSummary(mDecodeSupl, value);
        }
        mDecodeSupl.setOnPreferenceChangeListener(new OnPreferenceChangeListener() {
            @Override
            public boolean onPreferenceChange(Preference preference, Object objValue) {
                Editor editor = BaseSetting.getSharedPreferences(UpcSetting.this).edit();
                editor.putInt(KEY_DECODE_SUPL, new Integer((String) objValue));
                editor.commit();
                setParameter(DECODE_SUPL_PARM, new Integer((String) objValue));
                updateListPreferenceSummary((ListPreference)preference,
                        Long.parseLong((String) objValue));
                return true;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    protected void onPause() {
        super.onPause();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }
}
