package com.xcheng.scannere3.scantest;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.xcheng.scannere3.*;

public class ScanActivity extends Activity implements ScanListAdapter.OnShowItemClickListener, OnClickListener {

    private static final String TAG = "ScanActivity";

    private static final boolean mDbgLogEnable = false;

    public static final int MSG_SEND_RESULT = 0x01;

    private ListView mListView;
    private TextView mSelectNumView;
    private TextView mTotalNumView;
    private Button mExit;
    private List<ScanData> mDataList;
    private List<ScanData> mSelectList;
    private ScanListAdapter mAdapter;
    private ScanDatabaseControler mControler;
    private boolean mKeyDown = false;
    private boolean mIsFactoryTestingMode = false; // true: called by factory testing app.

/*
    private final HandlerThread mBeepHandlerThread = new HandlerThread("beep");

    {
        this.mBeepHandlerThread.start();
    }

    private final Handler mBeepHandler = new Handler(this.mBeepHandlerThread.getLooper()) {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            mBeep.play();
            Log.d(TAG, "mBeepHandler:handleMessage");
        }
    };
*/

    @SuppressLint("HandlerLeak")
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);

            switch(msg.what) {
                case MSG_SEND_RESULT:
                    if (mDbgLogEnable) {
                        Log.d(TAG, "handleMessage: MSG_SEND_RESULT");
                    }
                    ScanData data = (ScanData) msg.obj;
                    String barcode = data.getBarcode();
                    if (!TextUtils.isEmpty(barcode)) {
                        if (mDbgLogEnable) {
                            Log.d(TAG, "barcode: " + barcode);
                        }
                        
                        if (isInFactoryTestingMode()) {
                            sendBarcodeForResult(barcode);
                        }
                    }
                    break;

                default:
                    break;
            }

            if (msg.obj != null) {
                addData(mAdapter.getCount(), (ScanData) msg.obj);
            }
        }
    };

/*
    private BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            int num = mAdapter.getCount();

            // ScanTestService will send SCAN_DECODING_BROADCAST after got decode result
            if (action.equals(ScanUtil.SCAN_DECODING_BROADCAST)) {
                Log.d(TAG, "onReceive SCAN_DECODING_BROADCAST");
                
                String barcode = intent.getStringExtra(ScanUtil.SCAN_DECODING_DATA);
                String symbology = intent.getStringExtra(ScanUtil.SCAN_SYMBOLOGY_TYPE);
                ScanData scanData = new ScanData(symbology, barcode);
                
                Log.d(TAG, "num = " + num + ", symbology = " + symbology + ", barcode = " + barcode);

                if (isInFactoryTestingMode()) {
                    Message msgResult = Message.obtain();
                    msgResult.what = MSG_SEND_RESULT;
                    msgResult.arg1 = 1;
                    msgResult.obj = scanData;
                    mHandler.sendMessage(msgResult);
                } else {
                    addData(num, scanData);
                }
            }
        }
    };
*/

    private final XCScanner.Result mScanActivityResultListener = new XCScanner.Result(){
        @Override
        public void scanBeep() {
            if (mDbgLogEnable) {
                Log.d(TAG, "mScanContinuxActivityResultListener.scanBeep");
            }
        }

        @Override
        public void scanStart() {
            if (mDbgLogEnable) {
                Log.d(TAG, "mScanContinuxActivityResultListener.scanStart");
            }
        }

        @Override
        public void scanResult(String symbology,String barcode) {
            if (mDbgLogEnable) {
                Log.d(TAG, "mScanActivityResultListener.scanResult");
            }
            ScanData scanData = new ScanData(symbology, barcode);
            Message msgResult = Message.obtain();
            msgResult.what = MSG_SEND_RESULT;
            msgResult.arg1 = 1;
            msgResult.obj = scanData;
            mHandler.sendMessage(msgResult);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        Log.d(TAG, "onCreate +");
        super.onCreate(savedInstanceState);

        // Check if this activaty is called from factory testing app.
        mIsFactoryTestingMode = getIntent().getBooleanExtra(ScanUtil.FACTORY_SCAN_TEST, false);

        // If creat from factory testing app, should be 
        // - Full screen
        // - Keep screen on
        // - Hide NAV bar
        if (isInFactoryTestingMode()) {
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
            getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_GESTURE_ISOLATED
                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY
                    | View.SYSTEM_UI_FLAG_FULLSCREEN);

            if (!ScanTestService.isScanServiceRunning()) {
                startService(new Intent(ScanActivity.this, ScanTestService.class));
            }
        }

        //registerReceiver(mReceiver, new IntentFilter(ScanUtil.SCAN_DECODING_BROADCAST));
        setContentView(R.layout.activity_scan);

        initView();
        Log.d(TAG, "onCreate -");
    }

    private void initView() {
        mControler = new ScanDatabaseControler(this);
        mListView = (ListView) findViewById(R.id.scan_listview);
        mSelectNumView = (TextView) findViewById(R.id.list_selected_num);
        mTotalNumView = (TextView) findViewById(R.id.list_total_num);
        mDataList = new ArrayList<ScanData>();
        mSelectList = new ArrayList<ScanData>();
        mDataList = mControler.queryAll();
        mAdapter = new ScanListAdapter(this);
        mAdapter.setData(mDataList);
        mListView.setAdapter(mAdapter);
        int num = mAdapter.getCount();
        mTotalNumView.setText(String.valueOf(num));
        mSelectNumView.setText("0");
        mExit = (Button) this.findViewById(R.id.exit_button);
        mExit.setOnClickListener(this);

        // If it is not creat from factory testing app, the EXIT button should be invisible.
        if (!isInFactoryTestingMode()) {
            mExit.setVisibility(View.GONE);
        }
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume+");
        super.onResume();

        // Notify service: enter scan test activity
        Intent intent = new Intent(ScanUtil.ACTION_SCAN_TEST_ACTIVITY_ENTER);
        sendBroadcast(intent);

        if (isInFactoryTestingMode()) {
            // Notify service: ScanActivity is in factory mode
            intent = new Intent(ScanUtil.ACTION_SCAN_TEST);
            intent.putExtra(ScanUtil.FACTORY_SCAN_TEST, true);
            sendBroadcast(intent);
        }

        ScanTestService.setupScanServiceResultListener(mScanActivityResultListener);

        Log.d(TAG, "onResume-");
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause+");
        super.onPause();

        // Notify service: leave scan test activity
        Intent intent = new Intent(ScanUtil.ACTION_SCAN_TEST_ACTIVITY_LEAVE);
        sendBroadcast(intent);

        if (isInFactoryTestingMode()) {
            // Notify service: ScanActivity is not in factory mode
            intent = new Intent(ScanUtil.ACTION_SCAN_TEST);
            intent.putExtra(ScanUtil.FACTORY_SCAN_TEST, false);
            sendBroadcast(intent);
        }

        E3Util.stopDecode();
        ScanTestService.setupScanServiceResultListener(null);
        
        Log.d(TAG, "onPause-");
    }

    @Override
    protected void onDestroy() {
        Log.d(TAG, "onDestroy+");
        super.onDestroy();
        //if (mReceiver != null) {
        //    unregisterReceiver(mReceiver);
        //}
        mHandler.removeMessages(MSG_SEND_RESULT);
        mAdapter.removeAllData();
        mControler.deleteAll();

        E3Util.stopDecode();
        
        //this.mBeep.close();
        //this.mBeepHandlerThread.quitSafely();

        Log.d(TAG, "onDestroy-");
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        if (!isInFactoryTestingMode()) {
            getMenuInflater().inflate(R.menu.scan_test_option, menu);

            // Default hide light test menu 
            menu.getItem(3).setVisible(false);
            menu.getItem(4).setVisible(false);
            menu.getItem(5).setVisible(false);
            menu.getItem(6).setVisible(false);
            menu.getItem(7).setVisible(false);
            menu.getItem(8).setVisible(false);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        if (id == R.id.menu_select_all) {
            markAll(true);
            return true;
        } else if (id == R.id.menu_select_erase) {
            deleteData();
            return true;
        } else if (id == R.id.menu_select_release) {
            markAll(false);
            return true;
        } else if (id == R.id.menu_lighttest_torchon) {
            E3Util.setLightTestMode(XCScanner.LIGHT_TEST_MODE_TORCH_ON);
            return true;
        } else if (id == R.id.menu_lighttest_torchoff) {
            E3Util.setLightTestMode(XCScanner.LIGHT_TEST_MODE_TORCH_OFF);
            return true;
        } else if (id == R.id.menu_lighttest_flash) {
            E3Util.setLightTestMode(XCScanner.LIGHT_TEST_MODE_FLASH);
            return true;
        } else if (id == R.id.menu_lighttest_aimon) {
            E3Util.setLightTestMode(XCScanner.LIGHT_TEST_MODE_AIM_ON);
            return true;
        } else if (id == R.id.menu_lighttest_aimoff) {
            E3Util.setLightTestMode(XCScanner.LIGHT_TEST_MODE_AIM_OFF);
            return true;
        } else if (id == R.id.menu_lighttest_reset) {
            E3Util.setLightTestMode(XCScanner.LIGHT_TEST_MODE_RESET);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    public void addData(int position, ScanData scanData) {
        mAdapter.addData(position, scanData);
        mAdapter.notifyDataSetChanged();
        mListView.setSelection(position);
        mControler.insert(scanData);
        mTotalNumView.setText(String.valueOf(++position));
    }

    public void deleteData() {
        Map<Integer, Boolean> isCheck_delete = mAdapter.getMap();
        int count = mAdapter.getCount();
        int num = count;
        if (mAdapter.isSelectAll) {
            mAdapter.removeAllData();
            mControler.deleteAll();
            num = 0;
        } else {
            for (int i = 0; i < count; i++) {
                int position = i - (count - mAdapter.getCount());
                num = position;
                if (isCheck_delete.get(i) != null && isCheck_delete.get(i)) {
                    isCheck_delete.remove(i);
                    mControler.delete(mAdapter.getId(position));
                    mAdapter.removeData(position);
                }
            }
        }
        mTotalNumView.setText(String.valueOf(num));
        mSelectNumView.setText("0");
        mAdapter.initCheck(false);
        mAdapter.notifyDataSetChanged();
        refresh();
    }

    public void markAll(boolean select) {
        Map<Integer, Boolean> isCheck = mAdapter.getMap();
        mAdapter.initCheck(select);
        mAdapter.notifyDataSetChanged();
        mSelectNumView.setText(String.valueOf(mAdapter.selectNum));
    }

    public void refresh() {
        onCreate(null);
    }

    public boolean isInFactoryTestingMode() {
        if (mDbgLogEnable) {
            Log.d(TAG, "mIsFactoryTestingMode = " + mIsFactoryTestingMode);
        }
        
        return mIsFactoryTestingMode;
    }

    // Set Activaty return value for other package
    public void sendBarcodeForResult(String barcode) {
        Log.d(TAG, "sendBarcodeForResult");
        
        Intent intent = new Intent();
        intent.putExtra("result", barcode);
        setResult(RESULT_OK, intent);
        ScanUtil.openCloseDevice(getApplicationContext(), false);
        finish();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        Log.d(TAG, "onKeyDown keyCode = " + keyCode + ", event = " + event);
        if (keyCode == KeyEvent.KEYCODE_FUNCTION
                || keyCode == KeyEvent.KEYCODE_F11
                || keyCode == KeyEvent.KEYCODE_XC_LEFT_SCAN
                || keyCode == KeyEvent.KEYCODE_XC_RIGHT_SCAN) {
            if (!mKeyDown) {
                mKeyDown = true;

                // Ask scan service start scan
                //ScanUtil.openCloseDevice(getApplicationContext(), true);
                E3Util.doDecode();
            }
            return true;
        }

        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        Log.d(TAG, "onKeyUp keyCode = " + keyCode + ", event = " + event);
        if (keyCode == KeyEvent.KEYCODE_FUNCTION
                || keyCode == KeyEvent.KEYCODE_F11
                || keyCode == KeyEvent.KEYCODE_XC_LEFT_SCAN
                || keyCode == KeyEvent.KEYCODE_XC_RIGHT_SCAN) {
            if (mKeyDown) {
                mKeyDown = false;

                // Ask scan service stop scan
                //ScanUtil.openCloseDevice(getApplicationContext(), false);
                E3Util.stopDecode();
            }
            return true;
        }
        return super.onKeyUp(keyCode, event);
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        Log.d(TAG, "onClick id = " + id);
        switch (id) {
            case R.id.exit_button:
                onBackPressed();
                break;
        }

    }
}

